@@ -1,9 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
-# Multibin Syntax
-
-## Introduction
+## Multibin Syntax
 
 Many refinery units receive arguments which represent binary data, and usually these arguments can
 be given in **multibin** format, which is a special syntax which allows to preprocess data with a
@@ -41,7 +39,7 @@ The modifiers `s`, `u`, `h`, `copy` (or `c`), and `cut` (or `x`) along with usin
 should cover most use cases. To learn about other existing modifiers, refer to the rest of this
 documentation.
 
-## The Details
+## Technical Details
 
 This module implements all argument parser types for the binary refinery. Notable classes for the
 command line use are the following:
@@ -89,6 +87,7 @@ from typing import get_type_hints
 from ..lib.frame import Chunk
 from ..lib.tools import isbuffer, infinitize
 from ..lib.loader import resolve, EntryNotFound
+from ..lib.meta import metavars
 
 FinalType = TypeVar('FinalType')
 
@@ -109,6 +108,13 @@ class RepeatedInteger(int):
         while True: yield self
 
 
+class LazyEvaluation:
+    """
+    Empty parent class for any unit that throws `refinery.lib.argformats.TooLazy`.
+    """
+    pass
+
+
 class PythonExpression:
     """
     Implements a parser for any Python expression with a prescribed set of variable
@@ -151,17 +157,37 @@ class PythonExpression:
     def __str__(self):
         return self.definition
 
-    def __call__(self, **values):
-        values.update(self.constants)
+    def __call__(self, mapping=None, **values):
+        if mapping is not None:
+            values, tmp = mapping, values
+            values.update(tmp)
+        variables = {}
         for v in self.variables:
-            if v not in values:
-                raise VariableMissing(v)
-        return eval(self.definition, None, values)
+            try:
+                variables[v] = values[v]
+            except KeyError:
+                raise ParserVariableMissing(v)
+        variables.update(self.constants)
+        return eval(self.definition, None, variables)
 
     @classmethod
-    def evaluate(cls, definition, **values):
-        expression = cls(definition, *list(values))
-        return expression(**values)
+    def evaluate(cls, definition, values):
+        expression = cls(definition, all_variables_allowed=True)
+        for name in expression.variables:
+            if name not in values:
+                raise ParserVariableMissing(name)
+        return expression(values)
+
+
+class SliceAgain(LazyEvaluation):
+    def __init__(self, expr):
+        self.expr = expr
+
+    def __call__(self, data):
+        expression = self.expr
+        if pending(expression):
+            expression = expression(data).decode('utf8')
+        return sliceobj(expression, metavars(data))
 
 
 def sliceobj(expression: Union[int, str, slice], variables: Optional[dict] = None, range=False) -> slice:
@@ -171,25 +197,28 @@ def sliceobj(expression: Union[int, str, slice], variables: Optional[dict] = Non
     argument format type will process the string `0x11:0x11+4*0x34` as the slice
     object `slice(17, 225, None)`.
     """
-    expression = expression or ':'
     if isinstance(expression, slice):
         return expression
     if isinstance(expression, int):
-        sliced = (expression,)
-    else:
-        sliced = expression.split(':')
-        if not sliced or len(sliced) > 3:
-            raise ArgumentTypeError(F'the expression {expression} is not a valid slice.')
-        try:
-            kwargs = variables or {}
-            sliced = [None if not t else PythonExpression.evaluate(t, **kwargs) for t in sliced]
-        except ParserVariableMissing:
-            class SliceAgain(LazyEvaluation):
-                def __call__(self, data):
-                    return sliceobj(expression, data.meta)
-            if variables is not None:
-                raise
-            return SliceAgain()
+        return slice(expression, expression + 1)
+    try:
+        expression = DelayedBinaryArgument(expression)
+        expression = expression()
+    except TooLazy:
+        return SliceAgain(expression)
+
+    expression = expression.decode('utf8')
+    expression = expression or ':'
+    sliced = expression.split(':')
+    if not sliced or len(sliced) > 3:
+        raise ArgumentTypeError(F'the expression {expression} is not a valid slice.')
+    try:
+        kwargs = variables or {}
+        sliced = [None if not t else PythonExpression.evaluate(t, kwargs) for t in sliced]
+    except ParserVariableMissing:
+        if variables is not None:
+            raise
+        return SliceAgain(expression)
     if len(sliced) == 1:
         k = sliced[0]
         if not range:
@@ -236,13 +265,6 @@ class VariableMissing(ArgumentTypeError):
         self.name = name
 
 
-class LazyEvaluation:
-    """
-    Empty parent class for any unit that throws `refinery.lib.argformats.TooLazy`.
-    """
-    pass
-
-
 class DelayedArgumentDispatch:
     """
     This class is used as a decorator for the default handler of classes that inherit from
@@ -340,8 +362,8 @@ def LazyPythonExpression(expression: str) -> MaybeDelayedType[Any]:
     if parser.variables:
         def evaluate(data: Chunk):
             try:
-                return parser(**data.meta)
-            except VariableMissing:
+                return parser(metavars(data))
+            except ParserVariableMissing:
                 # It is possible that a byte string can accidentally look like a valid Python
                 # expression, e.g.: B0fGtH*9/HKlwT:
                 definition = parser.definition
@@ -450,14 +472,14 @@ class DelayedArgument(LazyEvaluation):
     def handler(self, expression: str) -> bytes:
         return self.default_handler(expression)
 
-    @handler.register('s', final=True)
+    @handler.register('s', 'S', final=True)
     def s(self, string: str) -> bytes:
         """
         The final modifier `s:string` returns the UTF-8 encoded representation of `string`.
         """
         return string.encode('UTF8')
 
-    @handler.register('u', final=True)
+    @handler.register('u', 'U', final=True)
     def u(self, string: str) -> bytes:
         """
         The final modifier `u:string` returns the UTF16 (little endian without BOM) encoded
@@ -465,14 +487,14 @@ class DelayedArgument(LazyEvaluation):
         """
         return string.encode('UTF-16LE')
 
-    @handler.register('a', final=True)
+    @handler.register('a', 'A', final=True)
     def a(self, string: str) -> bytes:
         """
         The final modifier `a:string` returns the latin-1 encoded representation of `string`.
         """
         return string.encode('LATIN-1')
 
-    @handler.register('H', 'h', final=True)
+    @handler.register('h', 'H', final=True)
     def h(self, string: str) -> bytes:
         """
         The final modifier `h:string` (or `H:string`) returns the hex decoding of `string`.
@@ -480,7 +502,7 @@ class DelayedArgument(LazyEvaluation):
         import base64
         return base64.b16decode(string, casefold=True)
 
-    @handler.register('f', 'file', final=True)
+    @handler.register('f', 'F', 'file', final=True)
     def file(self, path: str) -> bytes:
         """
         The final modifier `f:path` or `file:path` returns the contents of the file located
@@ -529,11 +551,7 @@ class DelayedArgument(LazyEvaluation):
         but the corresponding bytes are also removed from the input data.
         """
         def extract(data: Union[bytearray, Chunk]):
-            try:
-                meta = data.meta
-            except AttributeError:
-                meta = {}
-            bounds = sliceobj(region, meta)
+            bounds = sliceobj(region, metavars(data))
             result = bytearray(data[bounds])
             data[bounds] = []
             return result
@@ -557,8 +575,9 @@ class DelayedArgument(LazyEvaluation):
         The variable remains attached to the chunk.
         """
         def extract(data: Chunk):
+            meta = metavars(data)
             try:
-                result = data.meta[name]
+                result = meta[name]
             except KeyError:
                 raise VariableMissing(name)
             return self._interpret_variable(name, result)
@@ -579,7 +598,7 @@ class DelayedArgument(LazyEvaluation):
             return self._interpret_variable(name, result)
         return extract
 
-    @handler.register('e', 'eval')
+    @handler.register('e', 'E', 'eval')
     def eval(self, expression: Union[str, ByteString]) -> Any:
         """
         Final modifier `e:expression` or `eval:expression`; uses a `refinery.lib.argformats.PythonExpression`
@@ -677,7 +696,7 @@ class DelayedArgument(LazyEvaluation):
         to it. Slices are given in Python syntax, so `take[::2]` will extract every second item from
         the incoming data. The default sequence is `1:`, i.e. skipping the first element.
         """
-        bounds = bounds and sliceobj(bounds) or slice(1, None)
+        bounds = bounds and sliceobj(bounds, {}) or slice(1, None)
         try:
             return it[bounds]
         except TypeError:
@@ -712,16 +731,17 @@ class DelayedArgument(LazyEvaluation):
 
         def finalize(data: Optional[Chunk] = None):
             @lru_cache(maxsize=512, typed=False)
-            def accumulate(A): return update(A=A, **args)
-            args = data and data.meta or {}
-            A = seed and seed(**args) or 0
+            def accumulate(A):
+                return update(args, A=A)
+            args = metavars(data)
+            A = seed and seed(args) or 0
             while True:
                 yield A
                 A = accumulate(A)
 
         try:
             return finalize()
-        except VariableMissing:
+        except ParserVariableMissing:
             return finalize
 
     @handler.register('be')
@@ -759,12 +779,12 @@ class DelayedArgument(LazyEvaluation):
         reduction = PythonExpression(reduction, all_variables_allowed=True)
 
         def finalize(data: Optional[Chunk] = None):
-            args = data and data.meta or {}
-            return reduce(lambda S, B: reduction(S=S, B=B, **args), it, seed and seed(**args) or 0)
+            args = metavars(data)
+            return reduce(lambda S, B: reduction(args, S=S, B=B), it, seed and seed(args) or 0)
 
         try:
             return finalize()
-        except VariableMissing:
+        except ParserVariableMissing:
             return finalize
 
 
@@ -791,6 +811,10 @@ class DelayedNumSeqArgument(DelayedArgument):
     attempts to evalue the input as a Python expression.
     """
 
+    def __init__(self, expression: str, typecheck=True):
+        super().__init__(expression)
+        self.typecheck = typecheck
+
     def default_handler(self, expression: str) -> Iterable[int]:
         """
         Attempts to parse the input expression as a sequence of integers. If this fails, the handler defaults
@@ -830,6 +854,8 @@ class DelayedNumSeqArgument(DelayedArgument):
                 value = tmp
         if isinstance(value, int):
             return RepeatedInteger(value)
+        if not self.typecheck:
+            return value
         raise ArgumentTypeError(
             F'The value {value} computed from {self.expression} is of type {type(value).__name__} but the unit requested an '
             R'integer or a sequence of integers.'
@@ -984,13 +1010,13 @@ expressions.
 """
 
 
-def numseq(expression: Union[int, str]) -> Union[Iterable[int], DelayedNumSeqArgument]:
+def numseq(expression: Union[int, str], typecheck=True) -> Union[Iterable[int], DelayedNumSeqArgument]:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedNumSeqArgument`.
     """
     if isinstance(expression, int):
         return RepeatedInteger(expression)
-    arg = DelayedNumSeqArgument(expression)
+    arg = DelayedNumSeqArgument(expression, typecheck)
     with suppress(TooLazy):
         return arg()
     return arg
