@@ -45,9 +45,14 @@ class dnfields(PathExtractorUnit):
     def unpack(self, data):
         header = DotNetHeader(data, parse_resources=False)
         tables = header.meta.Streams.Tables
-        iwidth = len(str(len(tables.FieldRVA)))
+        fields = tables.FieldRVA
+        if not fields:
+            return
+        iwidth = len(str(len(fields)))
+        rwidth = max(len(F'{field.RVA:X}') for field in fields)
+        rwidth = max(rwidth, 4)
 
-        for k, rv in enumerate(tables.FieldRVA):
+        for k, rv in enumerate(fields):
             index = rv.Field.Index
             field = tables.Field[index - 1]
             fname = field.Name
@@ -75,11 +80,9 @@ class dnfields(PathExtractorUnit):
                 continue
             typename, count, size = guess
             totalsize = count * size
-            name = F'F{rv.RVA:08X}.{typename}[{count}]'
-            if fname.isprintable():
-                name = F'{name}:{fname}'
-            else:
-                fname = '<UNPRINTABLE>'
+            if not fname.isprintable():
+                fname = F'F{rv.RVA:0{rwidth}X}'
+            name = F'{fname}.{typename}[{count}]'
             self.log_info(lambda: F'field {k:0{iwidth}d} at RVA 0x{rv.RVA:04X} of type {typename}, count: {count}, name: {fname}')
             offset = header.pe.get_offset_from_rva(rv.RVA)
             yield UnpackResult(name, lambda t=offset, s=totalsize: data[t:t + s])
