@@ -11,15 +11,16 @@ class group(Unit):
         super().__init__(size=size)
 
     def process(self, data):
-        members = data.meta.pop('__grpm')
-        assert len(members) < self.args.size, F'received {len(members) + 1} items in group'
+        members = data.temp or ()
+        if len(members) >= self.args.size:
+            raise RuntimeError(F'received {len(members) + 1} items in group')
         yield data
         yield from members
 
-    def filter(self, inputs):
+    def filter(self, chunks):
         members = []
         header = None
-        for chunk in inputs:
+        for chunk in chunks:
             if not chunk.visible:
                 yield chunk
                 continue
@@ -27,8 +28,9 @@ class group(Unit):
                 yield header
                 header = None
             if header is None:
+                chunk.temp = members
                 header = chunk
-                header['__grpm'] = members = []
+                members.clear()
             else:
                 members.append(chunk)
         if header is not None:
