@@ -6,8 +6,7 @@ import codecs
 import string
 
 from . import arg, HexViewer
-from ...lib.meta import GetMeta, CustomStringRepresentation, SizeInt
-from ...lib.mime import FileMagicInfo
+from ...lib.meta import metavars, CustomStringRepresentation, SizeInt
 from ...lib.types import INF
 from ...lib.tools import isbuffer, lookahead
 
@@ -136,6 +135,8 @@ class peek(HexViewer):
 
         if data.temp:
             final, index = data.temp
+        if final and not index:
+            index = None
 
         if not self.args.brief:
             padding = 0
@@ -167,23 +168,17 @@ class peek(HexViewer):
                 return sepsize * '-'
             return F'--{title}' + '-' * (sepsize - len(title) - 2)
 
-        if index is None:
-            yield separator()
-            return
-
         if self.args.brief:
             final = False
         elif not self.args.bare:
-            meta = GetMeta(data)
-            meta['size'] = meta['size']
+            meta = metavars(data)
             if self.args.meta:
+                meta['size'] = meta['size']
+                if meta['magic'] == 'data':
+                    del meta['magic']
                 entropy_percent = meta['entropy'] * 100.0
                 meta['entropy'] = F'{entropy_percent:.2f}%'
                 meta['ic']
-            else:
-                meta.magic_info = FileMagicInfo(data[:0x1000])
-            if meta['magic'] == 'data':
-                del meta['magic']
             if index is not None:
                 meta['index'] = index
             yield from self._peekmeta(metrics.hexdump_width, separator(), **meta)
